import functools

from django.conf import settings
from django.middleware.locale import LocaleMiddleware
from django.urls import get_resolver, LocalePrefixPattern, Resolver404
from django.utils import translation


def resolve_class(get_resolver, path):
    path = str(path)
    tried = []
    match = get_resolver.pattern.match(path)
    if match:
        new_path, args, kwargs = match
        for pattern in get_resolver.url_patterns:
            try:
                sub_match = pattern.resolve(new_path)
            except Resolver404 as e:
                try:
                    get_resolver._extend_tried(tried, pattern, e.args[0].get('tried'))
                except AttributeError:
                    pass
            else:
                if sub_match:
                    sub_match_dict = {**kwargs, **get_resolver.default_kwargs}
                    sub_match_dict.update(sub_match.kwargs)
                    try:
                        get_resolver._extend_tried(tried, pattern, sub_match.tried)
                    except AttributeError:
                        pass
                    return pattern
                tried.append([pattern])


@functools.lru_cache(maxsize=None)
def is_language_prefix_patterns_used_for_url(urlconf, path):
    resolver_class = resolve_class(get_resolver(urlconf), path)
    if resolver_class and isinstance(resolver_class.pattern, LocalePrefixPattern):
        return True, resolver_class.pattern.prefix_default_language
    return False, False


class GarpixLocaleMiddleware(LocaleMiddleware):

    def process_request(self, request):
        urlconf = getattr(request, 'urlconf', settings.ROOT_URLCONF)

        i18n_patterns_used, prefixed_default_language = is_language_prefix_patterns_used_for_url(urlconf, request.path)
        language = translation.get_language_from_request(request, check_path=i18n_patterns_used)
        language_from_path = translation.get_language_from_path(request.path_info)

        if not language_from_path and i18n_patterns_used and not prefixed_default_language:
            language = settings.LANGUAGE_CODE
        translation.activate(language)
        request.LANGUAGE_CODE = translation.get_language()



from django.contrib import admin
from django.urls import path, re_path, include
from django.conf.urls.i18n import i18n_patterns
from multiurl import ContinueResolving, multiurl
from django.http import Http404
from django.conf import settings
from django.conf.urls.static import static
from garpix_user.views import LogoutView, LoginView
from garpix_page.views.page import PageView
from garpix_page.views.index import IndexView
from drf_spectacular.views import SpectacularAPIView, SpectacularSwaggerView

urlpatterns = [
    path('admin/', admin.site.urls),
    path('page_lock/', include('garpix_admin_lock.urls')),
    path('ckeditor/', include('ckeditor_uploader.urls')),
    path(f'{settings.API_URL}/social-auth/', include('rest_framework_social_oauth2.urls')),
    path('', include(('garpix_user.urls', 'garpix_user'), namespace='garpix_user')),
    path('', include(('garpix_page.urls', 'garpix_page'), namespace='garpix_page')),
]

urlpatterns += [
    path('logout/', LogoutView.as_view(url='/'), name="logout"),
    path('login/', LoginView.as_view(), name="authorize")
]


if settings.DEBUG or settings.ENABLE_SWAGGER:
    urlpatterns += [
        path(f'{settings.API_URL}/schema/', SpectacularAPIView.as_view(), name='schema'),
        path(f'{settings.API_URL}/docs/', SpectacularSwaggerView.as_view(url_name='schema'), name='swagger-ui'),
    ]

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT) + static(
    settings.STATIC_URL, document_root=settings.STATIC_ROOT)

urlpatterns += i18n_patterns(
    multiurl(
        path('', PageView.as_view()),
        re_path(r'^(?P<url>.*?)$', PageView.as_view(), name='page'),
        path('', IndexView.as_view()),
        catch=(Http404, ContinueResolving),
    ),
    prefix_default_language=settings.USE_DEFAULT_LANGUAGE_PREFIX,
)
